require("fizzx.fizz")
require("states")
require("player")

display:create("fsm", 800, 600, 32, true)

fizz.gravity = -500

-- platformer map
map =
{
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
  1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
}
maph = 16
mapw = #map/maph

scene = Layer(-16*16, 8*16)
display.viewport:add_child(scene)

-- create shapes and tile sprites
for x = 0, mapw - 1 do
  for y = 0, maph - 1 do
    local i = map[y*mapw + x + 1]
    if i == 1 then
      local wx, wy = x*16, -y*16
      local s = Sprite(wx, wy)
      scene:add_child(s)
      -- rect shape
      s.canvas:square(16)
      s.canvas:fill()
      fizz.addStatic('rect', wx, wy, 16, 16)
    end
  end
end

player = Player:create(32, -16*4)

sprite = Sprite()
scene:add_child(sprite)

tilesheet = Image()
tilesheet:load("player.png")

animations =
{
  standing = { start = 1, frames = 2, rate = 0.15 },
  running = { start = 3, frames = 6, rate = 0.1 },
  jumping = { start = 9, frames = 4, rate = 0.1 },
  falling = { start = 13, frames = 2, rate = 0.15 }
}

-- update screen
step = 0.016
accum = 0

timer = Timer()
timer:start(16, true)
timer.on_tick = function(timer)
  -- update the simulation
  local dt = timer:get_delta_ms()/1000
  accum = accum + dt
  while accum >= step do
    fizz.update(step)
    player:update(step)
    accum = accum - step
  end
  
  -- sync player position
  local x, y = player:getPosition()
  sprite:set_position(x, y)
  -- sync player direction
  local xm, ym = player:getMovement()
  if xm ~= 0 then
    sprite:set_scale(xm, 1)
  end
  -- sync player animation
  local a = animations[player.state]
  local frame = math.floor(player.elapsed/a.rate)
  frame = frame%(a.frames)
  frame = frame + a.start - 1
  local x = frame%4*16
  local y = math.floor(frame/4)*16
  local c = sprite.canvas
  c:clear()
  c:set_source_subimage(tilesheet, x, y, 16, 16)
  c:paint()
end
