Player = {}
PlayerMT = { __index = Player }

function Player:create(x, y)
  local self = {}
  setmetatable(self, PlayerMT)
  -- collision
  self.shape = fizz.addDynamic("rect", x, y, 8, 16)
  self.shape.damping = 0.5
  self.shape.xd, self.shape.yd = 0, 0
  function self.shape:onCollide(b, nx, ny, pen)
    -- add up displacement vectors
    self.xd = self.xd + nx*pen
    self.yd = self.yd + ny*pen
    return true
  end
  self.grounded = false
  -- state
  self.state = "falling"
  self.elapsed = 0
  return self
end
function Player:destroy()
  fizz.removeShape(self.shape)
  self.shape = nil
  self.grounded = nil
  self.state = nil
  self.elapsed = nil
end

function Player:setState(state, ...)
  assert(states[state], "invalid state")
  -- exit previous state
  if states[self.state].exit then
    states[self.state]:exit(self)
  end
  self.state = state
  self.elapsed = 0
  -- enter next state
  if states[self.state].enter then
    states[self.state]:enter(self, ...)
  end
end
function Player:updateState(dt)
  self.elapsed = self.elapsed + dt
  if states[self.state].update then
    states[self.state]:update(self, dt)
  end
end
function Player:message(msg, ...)
  if states[self.state].message then
    states[self.state]:message(self, msg, ...)
  end
end

function Player:getPosition()
  return self.shape.x, self.shape.y
end
function Player:setPosition(x, y)
  return fizz.positionShape(self.shape, x, y)
end

function Player:getVelocity()
  return fizz.getVelocity(self.shape)
end
function Player:setVelocity(vx, vy)
  return fizz.setVelocity(self.shape, vx, vy)
end

function Player:getMovement()
  local mx, my = 0, 0
  if love.keyboard.isDown("left") then
    mx = -1
  elseif love.keyboard.isDown("right") then
    mx = 1
  end
  if love.keyboard.isDown(" ") then
    my = -1
  end
  return mx, my
end

function Player:getDisplacement()
  return self.shape.xd, self.shape.yd
end
function Player:setDisplacement(xd, yd)
  self.shape.xd = xd
  self.shape.yd = yd
end

function Player:update(dt)
  -- current movement, velocity and displacement
  local xm, ym = self:getMovement()
  local xv, yv = self:getVelocity()
  local xd, yd = self:getDisplacement()

  local isgrounded = yd < 0
  local wasgrounded = self.grounded
  self.grounded = isgrounded
  
  if isgrounded and not wasgrounded then
    self:message("hitground")
  end
  if not isgrounded and wasgrounded then
    self:message("fall")
  end
  if yd > 0 and yv < 0 then
    self:message("hitroof")
  end
  if xm ~= 0 then
    self:message("move")
  end
  if ym < 0 then
    self:message("jump")
  end
  if xm == 0 then
    self:message("stop")
  end
  
  if isgrounded then
    self.shape.friction = 1
  else
    self.shape.friction = 0
  end
  
  self:updateState(dt)
  
  self:setDisplacement(0, 0)
end
